<?xml version="1.0"?>

<!--
    'Speek' XSLT stylesheet.
    
    A stylesheet for transforming Spit index files into a set of HTML
    files (indexes and slides).

    (c) 2004 Remko Troncon
-->

<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:exsl="http://exslt.org/common" extension-element-prefixes="exsl">

<xsl:output method="html" version="4.0" encoding="iso-8859-1" indent="yes"/>


<!-- Auxiliary variables -->
<xsl:variable name="stylesheet" 
    select="/index/generator[attribute::name='xslt']/settings/@stylesheet"/>
<xsl:variable name="linkup" 
    select="/index/generator[attribute::name='xslt']/settings/@linkup"/>
<xsl:variable name="lines_per_page" 
    select="/index/generator[attribute::name='xslt']/settings/@lines_per_page"/>
<xsl:variable name="thumbs_per_line" 
    select="/index/generator[attribute::name='xslt']/settings/@thumbs_per_line"/>
<xsl:variable name="thumbs_per_page" 
    select="$lines_per_page * $thumbs_per_line"/>
<xsl:variable name="images_prefix">
    <xsl:if test="string-length(/index/@html_dir) > 0">../</xsl:if>
</xsl:variable>
<xsl:variable name="originals_dir" 
              select="concat($images_prefix,/index/@originals_dir)"/>
<xsl:variable name="slides_dir" 
              select="concat($images_prefix,/index/@slides_dir)"/>
<xsl:variable name="thumbs_dir" 
              select="concat($images_prefix,/index/@thumbs_dir)"/>
    

<!-- Main template. This one generates all the HTML. -->
<xsl:template match="index">

    <!-- Generate the indexes -->
    <xsl:for-each select="picture[position() mod $thumbs_per_page = 1]">
        <!-- Write index. -->
        <xsl:choose>
            <xsl:when test="position() = 1">
                <exsl:document href="index.html">
                    <xsl:call-template name="index">
                        <xsl:with-param name="page" select="position()"/>
                    </xsl:call-template>
                </exsl:document>
            </xsl:when>
            <xsl:otherwise>
                <exsl:document href="page{position()}.html">
                    <xsl:call-template name="index">
                        <xsl:with-param name="page" select="position()"/>
                    </xsl:call-template>
                </exsl:document>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:for-each>


    <!-- Generate the slides -->
    <xsl:if test="/index/@slides_create=1">
        <xsl:for-each select="picture">
            <exsl:document href="slide{position()}.html">
                <xsl:call-template name="slide"/>
            </exsl:document>
        </xsl:for-each>
    </xsl:if>

</xsl:template>


<!-- ********************** Template for an index *********************** !-->

<xsl:template name="index">
    <xsl:param name="page"/>
    
    <!-- Auxiliary variables -->
    <xsl:variable name="first_pic"
        select="($page - 1) * $thumbs_per_page + 1"/>
    <xsl:variable name="pages"
        select="floor((count(/index/picture)-1) div $thumbs_per_page) + 1"/>
   

    <!-- Head -->
    <html> <head>
        <title> <xsl:value-of select="/index/@title"/> </title>
        <link rel="stylesheet" type="text/css" href="{$stylesheet}" />
    </head>

    <!-- Body -->
    <body>
    
    <!-- Title & date -->
    <h1> <xsl:value-of select="/index/@title"/> </h1>
    <h2> <xsl:value-of select="/index/@date"/> </h2>

    <div class="thumbnails">
        <!-- Thumb matrix -->
        <table>
        <xsl:for-each select="/index/picture[
                        position() &gt;= $first_pic and
                        position() &lt; $first_pic + $thumbs_per_page]">
            <xsl:if test="position() mod $thumbs_per_line = 1">
                <!-- FIXME: This is probably very lame -->
                <xsl:variable name="offset" select="$first_pic+position()-2"/>

                <!-- Row of images -->
                <tr>
                <xsl:for-each select=".|following-sibling::picture[position()>=1 
                                and position() &lt; $thumbs_per_line]">
                    <td>
                        <!-- The image -->
                        <xsl:variable name="image">
                            <img src="{$thumbs_dir}/{@name}" 
                                 alt="{@description}"/>
                        </xsl:variable>
                        
                        <xsl:choose>
                            <xsl:when test="/index/@slides_create=1">
                                <a href="slide{position()+$offset}.html">
                                    <xsl:copy-of select="$image"/>
                                </a>
                            </xsl:when>
                            <xsl:when test="/index/@originals_create=1">
                                <a href="{$originals_dir}/{@name}">
                                    <xsl:copy-of select="$image"/>
                                </a>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:copy-of select="$image"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </td>
                </xsl:for-each>
                </tr>

                <!-- Row of descriptions -->
                <tr class="descriptions">
                <xsl:for-each select=".|following-sibling::picture[position()>=1 
                                and position() &lt; $thumbs_per_line]">
                    <td class="descriptions"> 
                        <xsl:value-of select="@description"/> 
                    </td>
                </xsl:for-each>
                </tr>
            </xsl:if>
        </xsl:for-each>
        </table>

        
        <!-- Navigation -->
        <div class="navigation">
        <ul>
            <xsl:if test="$pages > 1">
                <!-- Link to previous page -->
                <xsl:choose>
                    <xsl:when test="$page = 2">
                        <li> <a href="index.html">&lt;</a> </li>
                    </xsl:when>
                    <xsl:otherwise>
                        <li> <a href="page{(($page + $pages - 2) 
                                         mod $pages) + 1}.html">&lt;</a> </li>
                    </xsl:otherwise>
                </xsl:choose>

                <!-- Random access link to other pages -->
                <li> <ul class="randomnav">
                <xsl:for-each select="/index/picture[position() 
                                        mod $thumbs_per_page = 1]">
                    <xsl:choose> 
                        <xsl:when test="position() = $page">
                            <li> <xsl:value-of select="position()"/> </li>
                        </xsl:when>
                        <xsl:when test="position() = 1">
                            <li> <a href="index.html"> 
                                <xsl:value-of select="position()"/>
                            </a> </li>
                        </xsl:when>
                        <xsl:otherwise>
                            <li> <a href="page{position()}.html">
                                <xsl:value-of select="position()"/>
                            </a> </li>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                </ul> </li> 
                            
                <!-- Link to next page -->
                <xsl:choose>
                    <xsl:when test="$page = $pages">
                        <li> <a href="index.html">&gt;</a> </li>
                    </xsl:when>
                    <xsl:otherwise>
                        <li> <a href="page{($page mod $pages)+1}.html">&gt;</a> 
                        </li>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>

            <br/> <br/>
            <xsl:if test="string-length($linkup) > 0">
                <li id="up"> <a href="{$linkup}">Up</a> </li>
            </xsl:if>
        </ul>
        </div>
    </div>
    </body>
    </html>
</xsl:template>



<!-- *********************** Template for a slide ************************ -->

<xsl:template name="slide">
   <!-- Head -->
    <html> <head>
        <title> <xsl:value-of select="/index/@title"/> 
                [<xsl:value-of select="@name"/>] </title>
            <link rel="stylesheet" type="text/css" href="{$stylesheet}"/>
    </head>

    <!-- Body -->
    <body>
        <!-- Title & date -->
        <h1> <xsl:value-of select="/index/@title"/> </h1>
        <h2> <xsl:value-of select="/index/@date"/> </h2>
    
        <div class="slide">
            <!-- Picture -->
            <a href="slide{(position() mod last()) + 1}.html">
                    <img src="{$slides_dir}/{@name}" 
                             alt="{@description}"/>
            </a>
            <p> <xsl:value-of select="@description"/> </p>

            <!-- Navigation -->
            <div class="navigation">
            <ul>
                <xsl:if test="last() > 1">
                    <!-- Link to previous page -->
                    <li id="previous">
                        <a href="slide{((position() + last() - 2) 
                                 mod last()) + 1}.html"> Previous </a>
                    </li> 
                    <!-- Link to next page -->
                    <li id="next">
                        <a href="slide{(position() 
                                 mod last()) + 1}.html"> Next </a>
                    </li>
                </xsl:if>

                <!-- Link to full-sized picture -->
                <xsl:if test="/index/@originals_create=1">
                    <li>
                        <a href="{$originals_dir}/{@name}"> Full Size </a>
                    </li>
                </xsl:if>

                <!-- Index link -->
                <li>
                    <xsl:variable name="page" select="floor((position()-1) 
                                                    div $thumbs_per_page) + 1"/>
                    <xsl:choose>
                        <xsl:when test="$page = 1">
                            <a href="index.html">Index</a>
                        </xsl:when>
                        <xsl:otherwise>
                            <a href="page{$page}.html">Index</a>
                        </xsl:otherwise>
                    </xsl:choose>
                </li>
            </ul>
            </div>
        </div>
    </body>
  </html>
</xsl:template>
</xsl:stylesheet>
